@@ -1442,6 +1442,8 @@ static void prepare_faceplates (WhirlUI* ui) {
 		} else if (val < 1) { \
 			snprintf (tmp, 24, "%.1f", val); \
 			memcpy (tmp, tmp + 1, strlen (tmp)); \
+		} else if (val < 3) { \
+			snprintf (tmp, 24, "%.0f", val); \
 		} else { \
 			snprintf (tmp, 24, "%.0f", 5. * rintf (val / 5)); \
 		} \
@@ -2432,7 +2434,7 @@ static RobWidget * toplevel (WhirlUI* ui, void * const top) {
 
 	ui->btn_adv = robtk_cbtn_new ("Unlock", GBT_LED_LEFT, false);
 
-	ui->lbl_adv[0] = robtk_lbl_new ("<markup><b>Advanced</b>\nChanging these\nparameters\nre-reinitializes\nthe DSP.\nExpect clickless\nshort fades.</markup>");
+	ui->lbl_adv[0] = robtk_lbl_new ("<markup><b>Advanced</b>\nChanging these\nparameters\nre-reinitializes\nthe DSP.\nAudio will fade\nduring reset.</markup>");
 	ui->lbl_adv[1] = robtk_lbl_new ("Mic \nDistance:");
 	ui->lbl_adv[2] = robtk_lbl_new ("Horn\nMic Angle:");
 	ui->lbl_adv[3] = robtk_lbl_new ("Horn\nRadius:");
