@@ -147,6 +147,14 @@ static void setIIRFilter (iir_t W[],
                           const double SR)
 {
 	double C[6];
+	if (Q <= 0.1) return;
+	if (Q >= 6.00) return;
+	if (F / SR <= 0.0002) return;
+	if (F / SR >= 0.4998) return;
+	if (G <= -48.0) return;
+	if (G >= 48.0) return;
+	if (T < EQC_LPF || T > EQC_HIGH) return;
+
 	eqCompute (T, F, Q, G, C, SR);
 	W[a1] = C[EQC_A1];
 	W[a2] = C[EQC_A2];
@@ -304,33 +312,7 @@ static void initTables (struct b_whirl *w) {
    * given coordinates are linearly interpolated and inserted into the
    * bfw matrix. Then (below) we normalise the lot to unit gain.
    */
-#if 1 // Fredrik
-  ipolmark (-180.0, 1.10);
-  ipoldraw (-166.3, 0.85, 0);
-  ipoldraw (-150.0, 0.85, 0);
-  ipoldraw (-134.8, 0.85, 0);
-  ipoldraw (-123.6, 0.78, 0);
-  ipoldraw (-105.6, 0.78, 0);
-  ipoldraw (-100.0, 0.78, 0);
-  ipoldraw ( -78.6, 0.85, 0);
-  ipoldraw ( -60.7, 0.81, 0);
-  ipoldraw ( -50.0, 0.89, 0);
-  ipoldraw ( -44.7, 0.90, 0);
-  ipoldraw ( -29.2, 1.30, 0);
-  ipoldraw (   0.0, 2.79, 0);
-  ipoldraw (  15.7, 2.30, 0);
-  ipoldraw (  31.5, 1.50, 0);
-  ipoldraw (  44.9, 0.90, 0);
-  ipoldraw (  50.0, 0.88, 0);
-  ipoldraw (  60.7, 0.80, 0);
-  ipoldraw (  74.2, 0.84, 0);
-  ipoldraw ( 100.0, 0.75, 0);
-  ipoldraw ( 121.3, 0.80, 0);
-  ipoldraw ( 150.0, 0.87, 0);
-  ipoldraw ( 166.3, 0.83, 0);
-  ipoldraw ( 180.0, 1.10, 0);
-#else // Robin
-  ipolmark (-180.0, 1.036);
+  ipolmark (-180.0, 1.052);
   ipoldraw (-166.4,  .881, 0);
   ipoldraw (-150.5,  .881, 0);
   ipoldraw (-135.3,  .881, 0);
@@ -354,99 +336,107 @@ static void initTables (struct b_whirl *w) {
   ipoldraw ( 120.0,  .836, 0);
   ipoldraw ( 135.3,  .836, 0);
   ipoldraw ( 150.0,  .881, 0);
-  ipoldraw ( 166.5,  .866, 0);
-  ipoldraw ( 180.0, 1.036, 0);
-#endif
+  ipoldraw ( 164.5,  .874, 0);
+  ipoldraw ( 180.0, 1.052, 0);
 
-  ipolmark (-180.0, -0.10);
+  ipolmark (-180.0, -0.07);
   ipoldraw (-150.0,  0.10, 1);
-  ipoldraw (-137.0, -0.10, 1);
-  ipoldraw (-118.0,  0.18, 1);
-  ipoldraw (-100.0,  0.20, 1);
-  ipoldraw ( -87.0,  0.40, 1);
-  ipoldraw ( -75.0,  0.30, 1);
+  ipoldraw (-135.0, -0.10, 1);
+  ipoldraw (-122.2,  0.16, 1);
+  ipoldraw (-105.0,  0.15, 1);
+  ipoldraw ( -91.2,  0.37, 1);
+  ipoldraw ( -75.3,  0.32, 1);
+  ipoldraw ( -60.1,  0.39, 1);
   ipoldraw ( -44.5,  0.70, 1);
-  ipoldraw ( -30.0,  0.45, 1);
-  ipoldraw (   3.1, -0.75, 1);
-  ipoldraw (  15.6, -0.51, 1);
-  ipoldraw (  35.7,  0.45, 1);
-  ipoldraw (  44.6,  0.67, 1);
-  ipoldraw (  74.7,  0.20, 1);
-  ipoldraw (  90.2,  0.31, 1);
-  ipoldraw ( 107.0,  0.02, 1);
-  ipoldraw ( 122.3,  0.15, 1);
-  ipoldraw ( 135.3, -0.12, 1);
+  ipoldraw ( -30.0,  0.53, 1);
+  ipoldraw ( -12.0, -0.40, 1);
+  ipoldraw (   0.0, -0.81, 1);
+  ipoldraw (   2.7, -0.77, 1);
+  ipoldraw (  15.0, -0.52, 1);
+  ipoldraw (  33.1,  0.38, 1);
+  ipoldraw (  43.7,  0.68, 1);
+  ipoldraw (  57.7,  0.49, 1);
+  ipoldraw (  74.1,  0.19, 1);
+  ipoldraw (  89.4,  0.33, 1);
+  ipoldraw ( 105.0,  0.03, 1);
+  ipoldraw ( 120.0,  0.12, 1);
+  ipoldraw ( 134.0, -0.13, 1);
   ipoldraw ( 153.3,  0.08, 1);
-  ipoldraw ( 180.0, -0.10, 1);
+  ipoldraw ( 180.0, -0.07, 1);
 
   ipolmark (-180.0,  0.40);
   ipoldraw (-165.0,  0.20, 2);
   ipoldraw (-150.0,  0.48, 2);
+  ipoldraw (-135.0,  0.27, 2);
   ipoldraw (-121.2,  0.22, 2);
   ipoldraw ( -89.2,  0.30, 2);
   ipoldraw ( -69.2,  0.22, 2);
   ipoldraw ( -58.0,  0.11, 2);
   ipoldraw ( -40.2, -0.43, 2);
-  ipoldraw ( -29.0, -0.58, 2);
-  ipoldraw ( -15.6, -0.49, 2);
+  ipoldraw ( -29.0, -0.53, 2);
+  ipoldraw ( -15.6, -0.43, 2);
   ipoldraw (   0.0,  0.00, 2);
-  ipoldraw (  17.8, -0.60, 2);
-  ipoldraw (  75.9,  0.35, 2);
-  ipoldraw (  91.5,  0.28, 2);
+  ipoldraw (  14.3, -0.44, 2);
+  ipoldraw (  30.3, -0.60, 2);
+  ipoldraw (  60.3,  0.11, 2);
+  ipoldraw (  74.9,  0.32, 2);
+  ipoldraw (  91.5,  0.23, 2);
   ipoldraw ( 104.9,  0.32, 2);
-  ipoldraw ( 122.7,  0.22, 2);
+  ipoldraw ( 121.7,  0.19, 2);
+  ipoldraw ( 135.0,  0.27, 2);
   ipoldraw ( 150.0,  0.45, 2);
-  ipoldraw ( 167.0,  0.20, 2);
+  ipoldraw ( 165.0,  0.20, 2);
   ipoldraw ( 180.0,  0.40, 2);
 
-  ipolmark (-180.0, -0.15);
-  ipoldraw (-165.2, -0.20, 3);
+  ipolmark (-180.0, -0.08);
+  ipoldraw (-165.2, -0.19, 3);
   ipoldraw (-150.0,  0.00, 3);
   ipoldraw (-133.9, -0.20, 3);
+  ipoldraw (-120.0, -0.15, 3);
   ipoldraw (-106.0,  0.09, 3);
-  ipoldraw ( -89.3, -0.20, 3);
+  ipoldraw ( -89.3, -0.15, 3);
   ipoldraw ( -76.3,  0.00, 3);
-  ipoldraw ( -60.3,  0.34, 3);
-  ipoldraw ( -44.6,  0.00, 3);
-  ipoldraw ( -15.6, -0.28, 3);
-  ipoldraw (   0.0,  0.36, 3);
-  ipoldraw (  14.5,  0.15, 3);
-  ipoldraw (  20.1, -0.18, 3);
-  ipoldraw (  44.6,  0.18, 3);
-  ipoldraw (  60.4,  0.26, 3);
-  ipoldraw (  75.9,  0.20, 3);
-  ipoldraw (  90.4, -0.10, 3);
-  ipoldraw ( 104.9,  0.10, 3);
-  ipoldraw ( 122.8, -0.10, 3);
-  ipoldraw ( 136.2, -0.10, 3);
-  ipoldraw ( 150.0,  0.12, 3);
-  ipoldraw ( 165.0, -0.25, 3);
-  ipoldraw ( 180.0, -0.15, 3);
+  ipoldraw ( -60.3,  0.29, 3);
+  ipoldraw ( -44.6, -0.02, 3);
+  ipoldraw ( -15.6, -0.22, 3);
+  ipoldraw (   0.0,  0.24, 3);
+  ipoldraw (  14.5,  0.11, 3);
+  ipoldraw (  30.1, -0.10, 3);
+  ipoldraw (  44.6,  0.17, 3);
+  ipoldraw (  60.4,  0.22, 3);
+  ipoldraw (  75.9,  0.16, 3);
+  ipoldraw (  90.4, -0.05, 3);
+  ipoldraw ( 104.9,  0.07, 3);
+  ipoldraw ( 122.8, -0.07, 3);
+  ipoldraw ( 136.2, -0.07, 3);
+  ipoldraw ( 150.0,  0.08, 3);
+  ipoldraw ( 165.0, -0.19, 3);
+  ipoldraw ( 180.0, -0.08, 3);
 
-  ipolmark (-180.0,  0.20);
+  ipolmark (-180.0,  0.13);
   ipoldraw (-165.2,  0.00, 4);
-  ipoldraw (-150.0,  0.22, 4);
-  ipoldraw (-136.2, -0.22, 4);
+  ipoldraw (-150.0,  0.17, 4);
+  ipoldraw (-135.2, -0.20, 4);
   ipoldraw (-120.5,  0.00, 4);
-  ipoldraw (-100.0,  0.00, 4);
-  ipoldraw ( -90.0,  0.10, 4);
-  ipoldraw ( -74.8, -0.15, 4);
-  ipoldraw ( -60.3, -0.20, 4);
-  ipoldraw ( -44.6,  0.21, 4);
+  ipoldraw (-105.0,  0.00, 4);
+  ipoldraw ( -90.0,  0.04, 4);
+  ipoldraw ( -75.0, -0.09, 4);
+  ipoldraw ( -60.3, -0.14, 4);
+  ipoldraw ( -45.0,  0.16, 4);
   ipoldraw ( -15.6,  0.00, 4);
-  ipoldraw (   0.0,  0.30, 4);
+  ipoldraw (   0.0,  0.22, 4);
   ipoldraw (  15.6, -0.21, 4);
-  ipoldraw (  20.1, -0.11, 4);
-  ipoldraw (  45.7,  0.18, 4);
-  ipoldraw (  60.3, -0.10, 4);
-  ipoldraw (  74.8, -0.20, 4);
-  ipoldraw (  90.4, -0.05, 4);
-  ipoldraw ( 104.9, -0.18, 4);
+  ipoldraw (  30.1, -0.09, 4);
+  ipoldraw (  45.0,  0.10, 4);
+  ipoldraw (  60.3, -0.07, 4);
+  ipoldraw (  74.8, -0.15, 4);
+  ipoldraw (  90.4, -0.03, 4);
+  ipoldraw ( 104.9, -0.14, 4);
   ipoldraw ( 120.5,  0.00, 4);
-  ipoldraw ( 136.2, -0.30, 4);
-  ipoldraw ( 150.0,  0.20, 4);
-  ipoldraw ( 165.0,  0.00, 4);
-  ipoldraw ( 180.0,  0.20, 4);
+  ipoldraw ( 135.2, -0.26, 4);
+  ipoldraw ( 150.0,  0.16, 4);
+  ipoldraw ( 165.0, -0.02, 4);
+  ipoldraw ( 180.0,  0.13, 4);
 
   sum = 0.0;
   /* Compute the normalisation factor */
@@ -1050,7 +1040,7 @@ void whirlProc2 (struct b_whirl *w,
 
 		if (w->hnBrakePos > 0 && w->hornTarget == 0 && w->hornIncr > 0 && w->hornIncr < hardstop) {
 			const double targetPos = fmod(1.25 - w->hnBrakePos, 1.0);
-			if (fabsf(w->hornAngleGRD - targetPos) < (2.0 / WHIRL_DISPLC_SIZE)) {
+			if (fabs(w->hornAngleGRD - targetPos) < (2.0 / WHIRL_DISPLC_SIZE)) {
 				w->hornAngleGRD = targetPos;
 				w->hornIncr = 0;
 			} else {
@@ -1087,7 +1077,7 @@ void whirlProc2 (struct b_whirl *w,
 
 		if (w->drBrakePos > 0 && w->drumTarget == 0 && w->drumIncr > 0 && w->drumIncr < hardstop) {
 			const double targetPos= fmod(w->drBrakePos + .75, 1.0);
-			if (fabsf(w->drumAngleGRD - targetPos) < (2.0 / WHIRL_DISPLC_SIZE)) {
+			if (fabs(w->drumAngleGRD - targetPos) < (2.0 / WHIRL_DISPLC_SIZE)) {
 				w->drumAngleGRD = targetPos;
 				w->drumIncr = 0;
 			} else {
@@ -1134,7 +1124,7 @@ void whirlProc2 (struct b_whirl *w,
 		const double targetPos = fmod(1.25 - w->hnBrakePos, 1.0);
 		if (!w->hornAcDc && w->hornIncr == 0 && w->hornAngleGRD != targetPos) {
 			brake_enagaged |= 1;
-			if (fabsf(w->hornAngleGRD - targetPos) < (2.0 / WHIRL_DISPLC_SIZE)) {
+			if (fabs(w->hornAngleGRD - targetPos) < (2.0 / WHIRL_DISPLC_SIZE)) {
 				/* provide a dead-zone for rounding */
 				w->hornAngleGRD = targetPos;
 			} else {
@@ -1148,7 +1138,7 @@ void whirlProc2 (struct b_whirl *w,
 		const double targetPos= fmod(w->drBrakePos + .75, 1.0);
 		if (!w->drumAcDc && w->drumIncr == 0 && w->drumAngleGRD != targetPos) {
 			brake_enagaged |= 2;
-			if (fabsf(w->drumAngleGRD - targetPos) < (2.0 / WHIRL_DISPLC_SIZE)) {
+			if (fabs(w->drumAngleGRD - targetPos) < (2.0 / WHIRL_DISPLC_SIZE)) {
 				w->drumAngleGRD = targetPos;
 			} else {
 				const float limit = 100.f /* RPM */ / (60. * w->SampleRateD);
