@@ -1,6 +1,7 @@
 /*
   Copyright 2012 David Robillard <http://drobilla.net>
   Copyright 2011-2012 Ben Loftis, Harrison Consoles
+  Copyright 2013,2015 Robin Gareus <robin@gareus.org>
 
   Permission to use, copy, modify, and/or distribute this software for any
   purpose with or without fee is hereby granted, provided that the above
@@ -31,11 +32,21 @@
 
 #include "pugl_internal.h"
 
+#ifdef WITH_SOFD
+#define HAVE_X11
+#include "../sofd/libsofd.h"
+#include "../sofd/libsofd.c"
+#endif
+
 /* work around buggy re-parent & focus issues on some systems
  * where no keyboard events are passed through even if the
  * app has mouse-focus and all other events are working.
  */
-#define XKEYFOCUSGRAB
+//#define XKEYFOCUSGRAB
+
+/* show messages during initalization
+ */
+//#define VERBOSE_PUGL
 
 struct PuglInternalsImpl {
 	Display*   display;
@@ -55,6 +66,7 @@ static int attrListSgl[] = {
 	GLX_GREEN_SIZE, 4,
 	GLX_BLUE_SIZE, 4,
 	GLX_DEPTH_SIZE, 16,
+	GLX_ARB_multisample, 1,
 	None
 };
 
@@ -68,15 +80,37 @@ static int attrListDbl[] = {
 	GLX_GREEN_SIZE, 4,
 	GLX_BLUE_SIZE, 4,
 	GLX_DEPTH_SIZE, 16,
+	GLX_ARB_multisample, 1,
 	None
 };
 
-PuglView*
+/**
+   Attributes for double-buffered RGBA with multi-sampling
+	 (antialiasing)
+*/
+static int attrListDblMS[] = {
+	GLX_RGBA,
+	GLX_DOUBLEBUFFER    , True,
+	GLX_RED_SIZE        , 4,
+	GLX_GREEN_SIZE      , 4,
+	GLX_BLUE_SIZE       , 4,
+	GLX_ALPHA_SIZE      , 4,
+	GLX_DEPTH_SIZE      , 16,
+	GLX_SAMPLE_BUFFERS  , 1,
+	GLX_SAMPLES         , 4,
+	None
+};
+
+	PuglView*
 puglCreate(PuglNativeWindow parent,
            const char*      title,
+           int              min_width,
+           int              min_height,
            int              width,
            int              height,
-           bool             resizable)
+           bool             resizable,
+           bool             ontop,
+           unsigned long    transientId)
 {
 	PuglView*      view = (PuglView*)calloc(1, sizeof(PuglView));
 	PuglInternals* impl = (PuglInternals*)calloc(1, sizeof(PuglInternals));
@@ -89,26 +123,50 @@ puglCreate(PuglNativeWindow parent,
 	view->impl   = impl;
 	view->width  = width;
 	view->height = height;
+	view->ontop  = ontop;
+	view->set_window_hints = true;
+	view->user_resizable = resizable;
 
 	impl->display = XOpenDisplay(0);
+	if (!impl->display) {
+		free(view);
+		free(impl);
+		return 0;
+	}
 	impl->screen  = DefaultScreen(impl->display);
+	impl->doubleBuffered = True;
+
+	XVisualInfo* vi = glXChooseVisual(impl->display, impl->screen, attrListDblMS);
+
+	if (!vi) {
+		vi = glXChooseVisual(impl->display, impl->screen, attrListDbl);
+#ifdef VERBOSE_PUGL
+		printf("puGL: multisampling (antialiasing) is not available\n");
+#endif
+	}
 
-	XVisualInfo* vi = glXChooseVisual(impl->display, impl->screen, attrListDbl);
 	if (!vi) {
 		vi = glXChooseVisual(impl->display, impl->screen, attrListSgl);
 		impl->doubleBuffered = False;
-		printf("singlebuffered rendering will be used, no doublebuffering available\n");
-	} else {
-		impl->doubleBuffered = True;
-		printf("doublebuffered rendering available\n");
+#ifdef VERBOSE_PUGL
+		printf("puGL: singlebuffered rendering will be used, no doublebuffering available\n");
+#endif
 	}
 
 	int glxMajor, glxMinor;
 	glXQueryVersion(impl->display, &glxMajor, &glxMinor);
-	printf("GLX-Version %d.%d\n", glxMajor, glxMinor);
+#ifdef VERBOSE_PUGL
+	printf("puGL: GLX-Version : %d.%d\n", glxMajor, glxMinor);
+#endif
 
 	impl->ctx = glXCreateContext(impl->display, vi, 0, GL_TRUE);
 
+	if (!impl->ctx) {
+		free(view);
+		free(impl);
+		return 0;
+	}
+
 	Window xParent = parent
 		? (Window)parent
 		: RootWindow(impl->display, impl->screen);
@@ -133,16 +191,30 @@ puglCreate(PuglNativeWindow parent,
 		0, 0, view->width, view->height, 0, vi->depth, InputOutput, vi->visual,
 		CWBorderPixel | CWColormap | CWEventMask, &attr);
 
+	if (!impl->win) {
+		free(view);
+		free(impl);
+		return 0;
+	}
+
 	XSizeHints sizeHints;
 	memset(&sizeHints, 0, sizeof(sizeHints));
-	if (!resizable) {
+	if (view->set_window_hints) {
 		sizeHints.flags      = PMinSize|PMaxSize;
-		sizeHints.min_width  = width;
-		sizeHints.min_height = height;
-		sizeHints.max_width  = width;
-		sizeHints.max_height = height;
+		sizeHints.min_width  = min_width;
+		sizeHints.min_height = min_height;
+		sizeHints.max_width  = resizable ? 2048 : width;
+		sizeHints.max_height = resizable ? 2048 : height;
+		if (min_width != width) {
+			sizeHints.flags |= PAspect;
+			sizeHints.min_aspect.x=min_width;
+			sizeHints.min_aspect.y=min_height;
+			sizeHints.max_aspect.x=min_width;
+			sizeHints.max_aspect.y=min_height;
+		}
 		XSetNormalHints(impl->display, impl->win, &sizeHints);
 	}
+	XResizeWindow(view->impl->display, view->impl->win, width, height);
 
 	if (title) {
 		XStoreName(impl->display, impl->win, title);
@@ -153,16 +225,33 @@ puglCreate(PuglNativeWindow parent,
 		XSetWMProtocols(impl->display, impl->win, &wmDelete, 1);
 	}
 
-	XMapRaised(impl->display, impl->win);
+	if (!parent && view->ontop) { /* TODO stay on top  */
+		Atom type = XInternAtom(impl->display, "_NET_WM_STATE_ABOVE", False);
+		XChangeProperty(impl->display, impl->win,
+				XInternAtom(impl->display, "_NET_WM_STATE", False),
+				XInternAtom(impl->display, "ATOM", False),
+				32, PropModeReplace, (unsigned char *)&type, 1);
+	}
+
+	if (transientId > 0) {
+		XSetTransientForHint(impl->display, impl->win, (Window)(transientId));
+	}
+
+	if (parent) {
+		XMapRaised(impl->display, impl->win);
+	}
 
 	if (glXIsDirect(impl->display, impl->ctx)) {
-		printf("DRI enabled\n");
+#ifdef VERBOSE_PUGL
+		printf("puGL: DRI enabled\n");
+#endif
 	} else {
-		printf("No DRI available\n");
+#ifdef VERBOSE_PUGL
+		printf("puGL: No DRI available\n");
+#endif
 	}
 
 	XFree(vi);
-
 	return view;
 }
 
@@ -172,12 +261,27 @@ puglDestroy(PuglView* view)
 	if (!view) {
 		return;
 	}
+#ifdef WITH_SOFD
+	x_fib_close(view->impl->display);
+#endif
 
+	//glXMakeCurrent(view->impl->display, None, NULL);
 	glXDestroyContext(view->impl->display, view->impl->ctx);
 	XDestroyWindow(view->impl->display, view->impl->win);
 	XCloseDisplay(view->impl->display);
 	free(view->impl);
 	free(view);
+	view = NULL;
+}
+
+PUGL_API void
+puglShowWindow(PuglView* view) {
+	XMapRaised(view->impl->display, view->impl->win);
+}
+
+PUGL_API void
+puglHideWindow(PuglView* view) {
+	XUnmapWindow(view->impl->display, view->impl->win);
 }
 
 static void
@@ -190,6 +294,7 @@ puglReshape(PuglView* view, int width, int height)
 	} else {
 		puglDefaultReshape(view, width, height);
 	}
+	glXMakeCurrent(view->impl->display, None, NULL);
 
 	view->width  = width;
 	view->height = height;
@@ -199,9 +304,12 @@ static void
 puglDisplay(PuglView* view)
 {
 	glXMakeCurrent(view->impl->display, view->impl->win, view->impl->ctx);
+#if 0
 	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 	glLoadIdentity();
+#endif
 
+	view->redisplay = false;
 	if (view->displayFunc) {
 		view->displayFunc(view);
 	}
@@ -210,8 +318,38 @@ puglDisplay(PuglView* view)
 	if (view->impl->doubleBuffered) {
 		glXSwapBuffers(view->impl->display, view->impl->win);
 	}
+	glXMakeCurrent(view->impl->display, None, NULL);
+}
 
-	view->redisplay = false;
+static void
+puglResize(PuglView* view)
+{
+	int set_hints = 1;
+	view->resize = false;
+	if (!view->resizeFunc) { return; }
+	/* ask the plugin about the new size */
+	view->resizeFunc(view, &view->width, &view->height, &set_hints);
+
+	XSizeHints *hints = XAllocSizeHints();
+	hints->min_width = view->width;
+	hints->min_height = view->height;
+	hints->max_width = view->user_resizable ? 2048 : view->width;
+	hints->max_height = view->user_resizable ? 2048 : view->height;
+	hints->flags = PMaxSize | PMinSize;
+
+	if (set_hints) {
+		XSetWMNormalHints(view->impl->display, view->impl->win, hints);
+	}
+	XResizeWindow(view->impl->display, view->impl->win, view->width, view->height);
+	XFlush(view->impl->display);
+	XFree(hints);
+
+#ifdef VERBOSE_PUGL
+	printf("puGL: window resize (%dx%d)\n", view->width, view->height);
+#endif
+
+	/* and call Reshape in glX context */
+	puglReshape(view, view->width, view->height);
 }
 
 static PuglKey
@@ -263,12 +401,42 @@ setModifiers(PuglView* view, unsigned xstate, unsigned xtime)
 	view->mods |= (xstate & Mod4Mask)    ? PUGL_MOD_SUPER  : 0;
 }
 
+Window x_fib_window();
+
 PuglStatus
 puglProcessEvents(PuglView* view)
 {
 	XEvent event;
 	while (XPending(view->impl->display) > 0) {
 		XNextEvent(view->impl->display, &event);
+
+#ifdef WITH_SOFD
+		if (x_fib_handle_events(view->impl->display, &event)) {
+			const int status = x_fib_status();
+
+			if (status > 0) {
+				char* const filename = x_fib_filename();
+				x_fib_close(view->impl->display);
+				x_fib_add_recent (filename, time(NULL));
+				//x_fib_save_recent ("~/.robtk.recent");
+				if (view->fileSelectedFunc) {
+					view->fileSelectedFunc(view, filename);
+				}
+				free(filename);
+				x_fib_free_recent ();
+			} else if (status < 0) {
+				x_fib_close(view->impl->display);
+				if (view->fileSelectedFunc) {
+					view->fileSelectedFunc(view, NULL);
+				}
+			}
+		}
+#endif
+
+		if (event.xany.window != view->impl->win) {
+			continue;
+		}
+
 		switch (event.type) {
 		case MapNotify:
 			puglReshape(view, view->width, view->height);
@@ -349,25 +517,35 @@ puglProcessEvents(PuglView* view)
 				}
 			}
 
-			if (!repeated && view->keyboardFunc) {
+			if (!repeated) {
 				KeySym sym = XLookupKeysym(&event.xkey, 0);
 				PuglKey special = keySymToSpecial(sym);
-				if (!special) {
-					view->keyboardFunc(view, false, sym);
-				} else if (view->specialFunc) {
-					view->specialFunc(view, false, special);
+#if 0 // close on 'Esc'
+				if (sym == XK_Escape && view->closeFunc) {
+					view->closeFunc(view);
+					view->redisplay = false;
+				} else
+#endif
+				if (view->keyboardFunc) {
+					if (!special) {
+						view->keyboardFunc(view, false, sym);
+					} else if (view->specialFunc) {
+						view->specialFunc(view, false, special);
+					}
 				}
 			}
 		} break;
-		case ClientMessage:
-			if (!strcmp(XGetAtomName(view->impl->display,
-			                         event.xclient.message_type),
-			            "WM_PROTOCOLS")) {
+		case ClientMessage: {
+			char* type = XGetAtomName(view->impl->display,
+			                          event.xclient.message_type);
+			if (!strcmp(type, "WM_PROTOCOLS")) {
 				if (view->closeFunc) {
 					view->closeFunc(view);
+					view->redisplay = false;
 				}
 			}
-			break;
+			XFree(type);
+		} break;
 #ifdef XKEYFOCUSGRAB
 		case EnterNotify:
 			XSetInputFocus(view->impl->display, view->impl->win, RevertToPointerRoot, CurrentTime);
@@ -378,6 +556,10 @@ puglProcessEvents(PuglView* view)
 		}
 	}
 
+	if (view->resize) {
+		puglResize(view);
+	}
+
 	if (view->redisplay) {
 		puglDisplay(view);
 	}
@@ -391,8 +573,32 @@ puglPostRedisplay(PuglView* view)
 	view->redisplay = true;
 }
 
+void
+puglPostResize(PuglView* view)
+{
+	view->resize = true;
+}
+
 PuglNativeWindow
 puglGetNativeWindow(PuglView* view)
 {
 	return view->impl->win;
 }
+
+int
+puglOpenFileDialog(PuglView* view, const char *title)
+{
+#ifdef WITH_SOFD
+	//x_fib_cfg_filter_callback (fib_filter_movie_filename);
+	if (x_fib_configure (1, title)) {
+		return -1;
+	}
+	//x_fib_load_recent ("~/.robtk.recent");
+	if (x_fib_show (view->impl->display, view->impl->win, 300, 300)) {
+		return -1;
+	}
+	return 0;
+#else
+	return -1;
+#endif
+}
