@@ -64,6 +64,7 @@ extern void rtk_osx_api_err(const char *msg);
 #include <string.h>
 #include <math.h>
 #include <time.h>
+#include <getopt.h>
 #include <assert.h>
 
 #if (defined _WIN32 && defined RTK_STATIC_INIT)
@@ -142,6 +143,25 @@ static jack_ringbuffer_t *rb_ctrl_from_ui = NULL;
 static jack_ringbuffer_t *rb_atom_to_ui = NULL;
 static jack_ringbuffer_t *rb_atom_from_ui = NULL;
 
+#ifdef HAVE_LIBLO
+#include <lo/lo.h>
+lo_server_thread          osc_server = NULL;
+static jack_ringbuffer_t *rb_osc_to_ui = NULL;
+
+typedef struct _osc_midi_event {
+	size_t size;
+	uint8_t buffer[3];
+} osc_midi_event_t;
+
+#ifndef OSC_MIDI_QUEUE_SIZE
+#define OSC_MIDI_QUEUE_SIZE (256)
+#endif
+
+static osc_midi_event_t event_queue[OSC_MIDI_QUEUE_SIZE];
+static int queued_events_start = 0;
+static int queued_events_end = 0;
+#endif
+
 static pthread_mutex_t gui_thread_lock = PTHREAD_MUTEX_INITIALIZER;
 static pthread_cond_t  data_ready = PTHREAD_COND_INITIALIZER;
 
@@ -178,14 +198,22 @@ struct DelayBuffer {
 	int c_dly; // current delay
 	int w_ptr;
 	int r_ptr;
-	float out_buffer[MAXPERIOD]; // TODO dynamically allocate, use jack-period
+	float out_buffer[MAXPERIOD];
 	float delay_buffer[MAXDELAY];
 };
 
+struct PValue {
+	uint32_t port_idx;
+	float    value;
+};
+
 struct LV2Port {
 	const char *name;
 	enum PortType porttype;
 	float val_default;
+	float val_min;
+	float val_max;
+	const char *doc;
 };
 
 typedef struct _RtkLv2Description {
@@ -243,6 +271,8 @@ static pthread_mutex_t       worker_lock = PTHREAD_MUTEX_INITIALIZER;
 static pthread_cond_t        worker_ready = PTHREAD_COND_INITIALIZER;
 static LV2_Worker_Interface* worker_iface = NULL;
 
+static pthread_mutex_t       port_write_lock = PTHREAD_MUTEX_INITIALIZER;
+
 static struct DelayBuffer **delayline = NULL;
 static uint32_t worst_capture_latency = 0;
 static uint32_t plugin_latency = 0;
@@ -328,6 +358,23 @@ delay_port (struct DelayBuffer *dly, uint32_t n_samples, float *in)
  */
 
 static int process (jack_nframes_t nframes, void *arg) {
+	if (nframes > MAXPERIOD) {
+		static bool warned_max_period = false;
+		if (!warned_max_period) {
+			warned_max_period = true;
+			fprintf (stderr, "Jack Period Size > %d is not supported (current %d)\n", MAXPERIOD , nframes);
+		}
+		if (inst->nports_midi_out > 0) {
+			void* buf = jack_port_get_buffer(midi_out, nframes);
+			jack_midi_clear_buffer(buf);
+		}
+		for (uint32_t i=0 ; i < inst->nports_audio_out; ++i) {
+			float * bp = (float*) jack_port_get_buffer (output_port[i], nframes);
+			memset (bp, 0, nframes * sizeof (float));
+		}
+		return 0;
+	}
+
 	while (jack_ringbuffer_read_space(rb_ctrl_from_ui) >= sizeof(uint32_t) + sizeof(float)) {
 		uint32_t idx;
 		jack_ringbuffer_read(rb_ctrl_from_ui, (char*) &idx, sizeof(uint32_t));
@@ -385,7 +432,7 @@ static int process (jack_nframes_t nframes, void *arg) {
 			uint32_t padded_size = ((sizeof(LV2_Atom_Event) + size) +  7) & (~7);
 
 			if (inst->min_atom_bufsiz > padded_size) {
-				printf("send time..\n");
+				//printf("send time..\n");
 				LV2_Atom_Event *aev = (LV2_Atom_Event *)seq;
 				aev->time.frames = 0;
 				aev->body.size   = size;
@@ -395,21 +442,44 @@ static int process (jack_nframes_t nframes, void *arg) {
 				seq +=  padded_size;
 			}
 		}
-		// TODO only if UI..?
-		while (jack_ringbuffer_read_space(rb_atom_from_ui) > sizeof(LV2_Atom)) {
-			LV2_Atom a;
-			jack_ringbuffer_read(rb_atom_from_ui, (char *) &a, sizeof(LV2_Atom));
-			uint32_t padded_size = atom_in->atom.size + a.size + sizeof(int64_t);
-			if (inst->min_atom_bufsiz > padded_size) {
-				memset(seq, 0, sizeof(int64_t)); // LV2_Atom_Event->time
-				seq += sizeof(int64_t);
-				jack_ringbuffer_read(rb_atom_from_ui, (char *) seq, a.size);
-				seq += a.size;
-				atom_in->atom.size += a.size + sizeof(int64_t);
+
+		if (gui_instance) {
+			while (jack_ringbuffer_read_space(rb_atom_from_ui) > sizeof(LV2_Atom)) {
+				LV2_Atom a;
+				jack_ringbuffer_read(rb_atom_from_ui, (char *) &a, sizeof(LV2_Atom));
+				uint32_t padded_size = atom_in->atom.size + a.size + sizeof(int64_t);
+				if (inst->min_atom_bufsiz > padded_size) {
+					memset(seq, 0, sizeof(int64_t)); // LV2_Atom_Event->time
+					seq += sizeof(int64_t);
+					jack_ringbuffer_read(rb_atom_from_ui, (char *) seq, a.size);
+					seq += a.size;
+					atom_in->atom.size += a.size + sizeof(int64_t);
+				}
 			}
 		}
+
 		if (inst->nports_midi_in > 0) {
-			/* inject midi events */
+#ifdef HAVE_LIBLO
+			/*inject OSC midi events, use time 0 */
+			while (queued_events_end != queued_events_start) {
+				uint32_t size = event_queue[queued_events_end].size;
+				uint32_t padded_size = ((sizeof(LV2_Atom_Event) + size) +  7) & (~7);
+
+				if (inst->min_atom_bufsiz > padded_size) {
+					LV2_Atom_Event *aev = (LV2_Atom_Event *)seq;
+					aev->time.frames = 0; // time
+					aev->body.size  = size;
+					aev->body.type  = uri_midi_MidiEvent;
+					memcpy(LV2_ATOM_BODY(&aev->body), event_queue[queued_events_end].buffer, size);
+					atom_in->atom.size += padded_size;
+					seq += padded_size;
+				}
+
+				queued_events_end = (queued_events_end + 1) % OSC_MIDI_QUEUE_SIZE;
+			}
+#endif
+
+			/* inject jack midi events */
 			void* buf = jack_port_get_buffer(midi_in, nframes);
 			for (uint32_t i = 0; i < jack_midi_get_event_count(buf); ++i) {
 				jack_midi_event_t ev;
@@ -474,20 +544,22 @@ static int process (jack_nframes_t nframes, void *arg) {
 	}
 
 	/* create port-events for change values */
-	// TODO only if UI..?
-	for (uint32_t p = 0; p < inst->nports_ctrl; p++) {
-		if (inst->ports[portmap_rctl[p]].porttype != CONTROL_OUT) continue;
 
-		if (plugin_ports_pre[p] != plugin_ports_post[p]) {
-#if 0
-			if (TODO this port reportsLatency) {
-				plugin_latency = rintf(plugin_ports_pre[p]);
-				jack_recompute_total_latencies(j_client);
-			}
+	if (gui_instance) {
+		for (uint32_t p = 0; p < inst->nports_ctrl; p++) {
+			if (inst->ports[portmap_rctl[p]].porttype != CONTROL_OUT) continue;
+
+			if (plugin_ports_pre[p] != plugin_ports_post[p]) {
+#if 0 // TODO regardless of UI.. use a dedicated port-ID
+				if (TODO this port reportsLatency) {
+					plugin_latency = rintf(plugin_ports_pre[p]);
+					jack_recompute_total_latencies(j_client);
+				}
 #endif
-			if (jack_ringbuffer_write_space(rb_ctrl_to_ui) >= sizeof(uint32_t) + sizeof(float)) {
-				jack_ringbuffer_write(rb_ctrl_to_ui, (char *) &portmap_rctl[p], sizeof(uint32_t));
-				jack_ringbuffer_write(rb_ctrl_to_ui, (char *) &plugin_ports_pre[p], sizeof(float));
+				if (jack_ringbuffer_write_space(rb_ctrl_to_ui) >= sizeof(uint32_t) + sizeof(float)) {
+					jack_ringbuffer_write(rb_ctrl_to_ui, (char *) &portmap_rctl[p], sizeof(uint32_t));
+					jack_ringbuffer_write(rb_ctrl_to_ui, (char *) &plugin_ports_pre[p], sizeof(float));
+				}
 			}
 		}
 	}
@@ -499,8 +571,7 @@ static int process (jack_nframes_t nframes, void *arg) {
 
 	/* Atom sequence port-events */
 	if (inst->nports_atom_out + inst->nports_midi_out > 0 && atom_out->atom.size > sizeof(LV2_Atom)) {
-		// TODO only if UI..?
-		if (jack_ringbuffer_write_space(rb_atom_to_ui) >= atom_out->atom.size + 2 * sizeof(LV2_Atom)) {
+		if (gui_instance && jack_ringbuffer_write_space(rb_atom_to_ui) >= atom_out->atom.size + 2 * sizeof(LV2_Atom)) {
 			LV2_Atom a = {atom_out->atom.size + (uint32_t) sizeof(LV2_Atom), 0};
 			jack_ringbuffer_write(rb_atom_to_ui, (char *) &a, sizeof(LV2_Atom));
 			jack_ringbuffer_write(rb_atom_to_ui, (char *) atom_out, a.size);
@@ -525,9 +596,14 @@ static int process (jack_nframes_t nframes, void *arg) {
 	}
 
 	/* wake up UI */
-	if (jack_ringbuffer_read_space(rb_ctrl_to_ui) > sizeof(uint32_t) + sizeof(float)
-			|| jack_ringbuffer_read_space(rb_atom_to_ui) > sizeof(LV2_Atom)
-			) {
+	if (gui_instance && (
+				   jack_ringbuffer_read_space(rb_ctrl_to_ui) >= sizeof(uint32_t) + sizeof(float)
+				|| jack_ringbuffer_read_space(rb_atom_to_ui) > sizeof(LV2_Atom)
+#ifdef HAVE_LIBLO
+				|| jack_ringbuffer_read_space(rb_osc_to_ui) >= sizeof(uint32_t) + sizeof(float)
+#endif
+			    )
+	   ) {
 		if (pthread_mutex_trylock (&gui_thread_lock) == 0) {
 			pthread_cond_signal (&data_ready);
 			pthread_mutex_unlock (&gui_thread_lock);
@@ -754,13 +830,23 @@ static void write_function(
 		fprintf(stderr, "LV2Host: write_function() unsupported buffer\n");
 		return;
 	}
-	if (port_index < inst->nports_total && portmap_ctrl[port_index] < 0) {
+	if (port_index >= inst->nports_total) {
+		fprintf(stderr, "LV2Host: write_function() invalid port\n");
+		return;
+	}
+	if (portmap_ctrl[port_index] < 0) {
 		fprintf(stderr, "LV2Host: write_function() unmapped port\n");
 		return;
 	}
+	if (inst->ports[port_index].porttype != CONTROL_IN) {
+		fprintf(stderr, "LV2Host: write_function() not a control input\n");
+		return;
+	}
 	if (jack_ringbuffer_write_space(rb_ctrl_from_ui) >= sizeof(uint32_t) + sizeof(float)) {
+		pthread_mutex_lock (&port_write_lock);
 		jack_ringbuffer_write(rb_ctrl_from_ui, (char *) &portmap_ctrl[port_index], sizeof(uint32_t));
 		jack_ringbuffer_write(rb_ctrl_from_ui, (char *) buffer, sizeof(float));
+		pthread_mutex_unlock (&port_write_lock);
 	}
 }
 
@@ -830,6 +916,164 @@ lv2_worker_schedule(LV2_Worker_Schedule_Handle unused,
 	return LV2_WORKER_SUCCESS;
 }
 
+/******************************************************************************
+ * OSC
+ */
+#ifdef HAVE_LIBLO
+
+static void osc_queue_midi_event (osc_midi_event_t *ev) {
+	if (((queued_events_start + 1) % OSC_MIDI_QUEUE_SIZE) == queued_events_end) {
+		return;
+	}
+	memcpy (&event_queue[queued_events_start], ev, sizeof(osc_midi_event_t));
+	queued_events_start = (queued_events_start + 1) % OSC_MIDI_QUEUE_SIZE;
+}
+
+static void oscb_error (int num, const char *m, const char *path) {
+	fprintf(stderr, "liblo server error %d in path %s: %s\n", num, path, m);
+}
+
+#define MIDI_Q3(STATUS)                                 \
+	osc_midi_event_t ev;                            \
+	ev.size = 3;                                    \
+	ev.buffer[0] = STATUS | (argv[0]->i & 0x0f);    \
+	ev.buffer[1] = argv[1]->i & 0x7f;               \
+	ev.buffer[2] = argv[2]->i & 0x7f;               \
+	osc_queue_midi_event (&ev);
+
+static int oscb_noteon (const char *path, const char *types, lo_arg **argv, int argc, lo_message msg, void *user_data) {
+	MIDI_Q3(0x90);
+	return 0;
+}
+
+static int oscb_noteoff (const char *path, const char *types, lo_arg **argv, int argc, lo_message msg, void *user_data) {
+	MIDI_Q3(0x80);
+	return 0;
+}
+
+static int oscb_cc (const char *path, const char *types, lo_arg **argv, int argc, lo_message msg, void *user_data) {
+	MIDI_Q3(0xb0);
+	return 0;
+}
+
+static int oscb_pc (const char *path, const char *types, lo_arg **argv, int argc, lo_message msg, void *user_data) {
+	osc_midi_event_t ev;
+	ev.size = 2;
+	ev.buffer[0] = 0xc0 | (argv[0]->i & 0x0f);
+	ev.buffer[1] = argv[1]->i & 0x7f;
+	ev.buffer[2] = 0x00;
+	osc_queue_midi_event (&ev);
+	return 0;
+}
+
+static int oscb_rawmidi (const char *path, const char *types, lo_arg **argv, int argc, lo_message msg, void *user_data) {
+	osc_midi_event_t ev;
+	ev.size = 3;
+	ev.buffer[0] = argv[0]->m[1];
+	ev.buffer[1] = argv[0]->m[2] & 0x7f;
+	ev.buffer[2] = argv[0]->m[3] & 0x7f;
+	osc_queue_midi_event (&ev);
+	return 0;
+}
+
+static int oscb_parameter (const char *path, const char *types, lo_arg **argv, int argc, lo_message msg, void *user_data) {
+	assert (argc == 2 && !strcmp (types, "if"));
+
+	const uint32_t port = argv[0]->i;
+	const float val = argv[1]->f;
+
+	if (inst->nports_ctrl <= port) {
+		fprintf (stderr, "OSC: Invalid Parameter 0 <= %d < %d\n", port, inst->nports_ctrl);
+		return 0;
+	}
+
+	const uint32_t port_index = portmap_rctl[port];
+
+	if (inst->ports[port_index].porttype != CONTROL_IN) {
+		fprintf (stderr, "OSC: mapped port (%d) is not a control input.\n", port_index);
+		return 0;
+	}
+
+	if (inst->ports[port_index].val_min < inst->ports[port_index].val_max) {
+		if (val < inst->ports[port_index].val_min || val > inst->ports[port_index].val_max) {
+			fprintf (stderr, "OSC: Value out of bounds %f <= %f <= %f\n",
+					inst->ports[port_index].val_min, val, inst->ports[port_index].val_max);
+			return 0;
+		}
+	}
+
+	//fprintf (stdout, "OSC: %d,  %d -> %f\n", port, port_index, val);
+
+	write_function (NULL, port_index, sizeof (float), 0, (const void*) &val);
+
+	if (jack_ringbuffer_write_space (rb_osc_to_ui) >= sizeof(uint32_t) + sizeof(float)) {
+		jack_ringbuffer_write (rb_osc_to_ui, (char *) &port_index, sizeof(uint32_t));
+		jack_ringbuffer_write (rb_osc_to_ui, (char *) &val, sizeof(float));
+	}
+
+	return 0;
+}
+
+struct osc_command {
+	const char *path;
+	const char *typespec;
+	lo_method_handler handler;
+	const char *documentation;
+};
+
+static struct osc_command OSCC[] = {
+	{"/x42/parameter",    "if",  &oscb_parameter, "Set Control Input value. control-port, value"},
+	{"/x42/midi/raw",     "m",   &oscb_rawmidi,   "Send raw midi message to plugin"},
+	{"/x42/midi/cc",      "iii", &oscb_cc,        "Send midi control-change: channel, parameter, value"},
+	{"/x42/midi/pc",      "ii",  &oscb_pc,        "Send midi program-change: channel, program"},
+	{"/x42/midi/noteon",  "iii", &oscb_noteon,    "Send midi note on: channel, key, velocity"},
+	{"/x42/midi/noteoff", "iii", &oscb_noteoff,   "Send midi note off: channel, key, velocity"},
+};
+
+static void print_oscdoc (void) {
+	printf("# X42 OSC methods. Format:\n");
+	printf("# Path <space> Typespec <space> Documentation <newline>\n");
+	printf("#######################################################\n");
+	for (size_t i = 0; i < sizeof(OSCC) / sizeof(struct osc_command); ++i) {
+		printf("%s %s %s\n", OSCC[i].path, OSCC[i].typespec, OSCC[i].documentation);
+	}
+}
+
+static int start_osc_server (int osc_port) {
+	char tmp[8];
+	uint32_t port = (osc_port > 100 && osc_port < 60000) ? osc_port : 9988;
+
+	snprintf(tmp, sizeof(tmp), "%d", port);
+	osc_server = lo_server_thread_new (tmp, oscb_error);
+
+	if (!osc_server) {
+		fprintf(stderr, "OSC failed to listen on port %s", tmp);
+		return -1;
+	} else {
+		char *urlstr = lo_server_thread_get_url (osc_server);
+		fprintf(stderr, "OSC server: %s\n", urlstr);
+		free (urlstr);
+	}
+
+	for (size_t i = 0; i < sizeof(OSCC) / sizeof(struct osc_command); ++i) {
+		lo_server_thread_add_method (osc_server, OSCC[i].path, OSCC[i].typespec, OSCC[i].handler, NULL);
+	}
+
+	lo_server_thread_start(osc_server);
+
+	return 0;
+}
+
+static void stop_osc_server () {
+	if (!osc_server) return;
+	lo_server_thread_stop (osc_server);
+	lo_server_thread_free (osc_server);
+	fprintf(stderr, "OSC server shut down.\n");
+	osc_server = NULL;
+}
+#endif
+
+
 /******************************************************************************
  * MAIN
  */
@@ -851,6 +1095,10 @@ static void cleanup(int sig) {
 		jack_ringbuffer_free(worker_responses);
 	}
 
+#ifdef HAVE_LIBLO
+	stop_osc_server ();
+#endif
+
 	if (plugin_dsp && plugin_instance && plugin_dsp->deactivate) {
 		plugin_dsp->deactivate(plugin_instance);
 	}
@@ -867,6 +1115,10 @@ static void cleanup(int sig) {
 	jack_ringbuffer_free(rb_atom_to_ui);
 	jack_ringbuffer_free(rb_atom_from_ui);
 
+#ifdef HAVE_LIBLO
+	jack_ringbuffer_free(rb_osc_to_ui);
+#endif
+
 	free(input_port);
 	free(output_port);
 
@@ -883,12 +1135,24 @@ static void cleanup(int sig) {
 	free(portmap_a_out);
 	free(portmap_ctrl);
 	free(portmap_rctl);
+	free(atom_in);
+	free(atom_out);
 	free_uri_map();
 	fprintf(stderr, "bye.\n");
 }
 
 static void run_one(LV2_Atom_Sequence *data) {
 
+#ifdef HAVE_LIBLO
+	while (jack_ringbuffer_read_space(rb_osc_to_ui) >= sizeof(uint32_t) + sizeof(float)) {
+		uint32_t idx;
+		float val;
+		jack_ringbuffer_read(rb_osc_to_ui, (char*) &idx, sizeof(uint32_t));
+		jack_ringbuffer_read(rb_osc_to_ui, (char*) &val, sizeof(float));
+		plugin_gui->port_event(gui_instance, idx, sizeof(float), 0, &val);
+	}
+#endif
+
 	while (jack_ringbuffer_read_space(rb_ctrl_to_ui) >= sizeof(uint32_t) + sizeof(float)) {
 		uint32_t idx;
 		float val;
@@ -985,13 +1249,13 @@ static void list_plugins (void) {
 static void print_usage (void) {
 #ifdef X42_MULTIPLUGIN
 	printf ("x42-%s - JACK %s\n\n", APPNAME, X42_MULTIPLUGIN_NAME);
-	printf ("Usage: x42-%s [ Plugin-ID or URI ]\n\n", APPNAME);
+	printf ("Usage: x42-%s [ OPTIONS ] [ Plugin-ID or URI ]\n\n", APPNAME);
 	printf ("This is a standalone JACK application of a collection of LV2 plugins.\n"
 		"Use ID -1, -l or --list for a dedicated list of included plugins.\n"
 		"By default the first listed plugin (ID 0) is used.\n\n");
 	printf ("List if available plugins: (ID \"Name\" URI)\n");
 	list_plugins();
-	printf ("\nSee also: <%s>\n", X42_MULTIPLUGIN_URI);
+
 #else
 
 #if defined X42_PLUGIN_STRUCT
@@ -1002,10 +1266,35 @@ static void print_usage (void) {
 	const LV2_Descriptor* d = inst->lv2_descriptor(inst->dsp_descriptor_id);
 
 	printf ("x42-%s - JACK %s\n\n", APPNAME, inst->plugin_human_id);
-	printf ("Usage: x42-%s\n\n", APPNAME);
+	printf ("Usage: x42-%s [ OPTIONS ]\n\n", APPNAME);
 	printf ("This is a standalone JACK application of the LV2 plugin:\n"
 	        "\"%s\".\n", inst->plugin_human_id);
+#endif
 
+	printf ("\nUsage:\n"
+"All control elements are operated in using the mouse:\n"
+" Click+Drag     left/down: decrease, right/up: increase value. Hold the Ctrl key to increase sensitivity.\n"
+" Shift+Click    reset to default value\n"
+" Scroll-wheel    up/down by 1 step (smallest possible adjustment for given setting). Rapid continuous scrolling increases the step-size.\n"
+"The application can be closed by sending a SIGTERM (CTRL+C) on the command-line of by closing the window.\n"
+			);
+
+	printf ("\nOptions:\n"
+" -h, --help                Display this help and exit.\n"
+#ifdef X42_MULTIPLUGIN
+" -l, --list                Print list of available plugins and exit.\n"
+#endif
+" -O <port>, --osc <port>   Listen for OSC messages on the given UDP port.\n"
+" -p <idx>:<val>, --port <idx>:<val>\n"
+"                           Set initial value for given control port.\n"
+" -P, --portlist            Print control port list on startup.\n"
+" --osc-doc                 Print available OSC commands and exit.\n"
+" -V, --version             Print version information and exit.\n"
+			);
+
+#ifdef X42_MULTIPLUGIN_URI
+	printf ("\nSee also: <%s>\n", X42_MULTIPLUGIN_URI);
+#else
 	printf ("\nSee also: <%s>\n", d->URI);
 #endif
 	printf ("Website: <http://x42-plugins.com/>\n");
@@ -1019,46 +1308,133 @@ static void print_version (void) {
 		"warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\n");
 }
 
+static void dump_control_ports (void) {
+	printf ("# Input Control Port List (%d ports)\n", inst->nports_ctrl);
+	for (uint32_t i = 0; i < inst->nports_ctrl; ++i) {
+		const uint32_t pi = portmap_rctl[i];
+		if (inst->ports[pi].porttype != CONTROL_IN) {
+			continue;
+		}
+		printf ("%3d: %s", i, inst->ports[pi].name);
+
+		if (inst->ports[pi].val_min < inst->ports[pi].val_max) {
+			printf (" (min: %.2f, max: %.2f)", inst->ports[pi].val_min, inst->ports[pi].val_max);
+		}
+
+		if (inst->ports[pi].val_default != nan) {
+			printf (" [default: %.2f]", inst->ports[pi].val_default);
+		}
+
+		if (inst->ports[pi].doc) {
+			printf (" -- %s", inst->ports[pi].doc);
+		}
+		printf ("\n");
+	}
+	printf ("### End Port List\n");
+}
+
 int main (int argc, char **argv) {
+	int c;
 	int rv = 0;
+	int osc_port = 0;
+	bool dump_ports = false;
 	uint32_t c_ain  = 0;
 	uint32_t c_aout = 0;
 	uint32_t c_ctrl = 0;
+	uint32_t n_pval = 0;
+	struct PValue *pval  = NULL;
 
-	if (argc > 2) {
-		print_usage();
-		return -1;
-	}
-	if (argc > 1 && (!strcmp(argv[1], "-h") || !strcmp(argv[1], "--help"))) {
-		print_usage();
-		return 0;
-	}
-	if (argc > 1 && (!strcmp(argv[1], "-V") || !strcmp(argv[1], "--version"))) {
-		print_version();
-		return 0;
+	const struct option long_options[] = {
+		{ "help",       no_argument,       0, 'h' },
+		{ "list",       no_argument,       0, 'l' },
+		{ "osc",        required_argument, 0, 'O' },
+		{ "osc-doc",    no_argument,       0,  0x100 },
+		{ "port",       required_argument, 0, 'p' },
+		{ "portlist",   no_argument,       0, 'P' },
+		{ "version",    no_argument,       0, 'V' },
+	};
+
+	const char *optstring = "hlO:p:PV1";
+
+	if (optind < argc && !strncmp (argv[optind], "-psn_0", 6)) {++optind;}
+
+	while ((c = getopt_long(argc, argv, optstring, long_options, NULL)) != -1) {
+		char *tmp;
+		switch (c) {
+			case 'h':
+				print_usage();
+				return 0;
+				break;
+			case '1': // catch -1, backward compat
+			case 'l':
+#ifdef X42_MULTIPLUGIN
+				list_plugins();
+#endif
+				return 0;
+				break;
+			case 'O':
+				osc_port = atoi (optarg);
+#ifndef HAVE_LIBLO
+				fprintf (stderr, "This version was compiled without OSC support.\n");
+#endif
+				break;
+			case 'p':
+				if ((tmp = strchr (optarg, ':')) != NULL && *(++tmp)) {
+					pval = (struct PValue*) realloc (pval, (n_pval + 1) * sizeof (struct PValue));
+					pval[n_pval].port_idx = atoi (optarg);
+					pval[n_pval].value = atof (tmp);
+					++n_pval;
+				}
+				break;
+			case 'P':
+				dump_ports = true;
+				break;
+			case 'V':
+				print_version();
+				return 0;
+				break;
+
+			case 0x100:
+#ifndef HAVE_LIBLO
+				fprintf (stderr, "This version was compiled without OSC support.\n");
+#else
+				print_oscdoc();
+#endif
+				return 0;
+				break;
+			default:
+				// silently ignore additional session options on OSX
+#ifndef __APPLE__
+				fprintf (stderr, "invalid argument.\n");
+				print_usage();
+				return(1);
+#endif
+				break;
+		}
 	}
 
 	// TODO autoconnect option.
+	// TODO allow to set initial params
 
 #ifdef X42_MULTIPLUGIN
-	if (argc > 1 && (atoi(argv[1]) < 0 || !strcmp(argv[1], "-l") || !strcmp(argv[1], "--list"))) {
+	inst = NULL;
+	if (optind < argc && atoi (argv[optind]) < 0) {
 		list_plugins();
 		return 0;
-	}
 
-	inst = NULL;
-	if (argc > 1 && strlen(argv[1]) > 2 && atoi(argv[1]) == 0) {
+	}
+	if (optind < argc && strlen(argv[optind]) > 2 && atoi (argv[optind]) == 0) {
 		unsigned int i;
 		for (i = 0; i < sizeof(_plugins) / sizeof(RtkLv2Description); ++i) {
 			const LV2_Descriptor* d = _plugins[i].lv2_descriptor(_plugins[i].dsp_descriptor_id);
-			if (strstr(d->URI, argv[1]) || strstr(_plugins[i].plugin_human_id, argv[1])) {
+			if (strstr(d->URI, argv[optind]) || strstr(_plugins[i].plugin_human_id, argv[optind])) {
 				inst = &_plugins[i];
 				break;
 			}
 		}
 	}
-	if (argc > 1 && !inst && atoi(argv[1]) >= 0) {
-		unsigned int plugid = atoi(argv[1]);
+	if (optind < argc && !inst && atoi (argv[optind]) >= 0) {
+		unsigned int plugid = atoi (argv[optind]);
 		if (plugid < (sizeof(_plugins) / sizeof(RtkLv2Description))) {
 			inst = &_plugins[plugid];
 		}
@@ -1078,7 +1454,7 @@ int main (int argc, char **argv) {
 
 #ifdef USE_WEAK_JACK
 	if (have_libjack()) {
-		fprintf(stderr, "JACK is not available. http://jackaudio.org/\n");
+		fprintf (stderr, "JACK is not available. http://jackaudio.org/\n");
 #ifdef _WIN32
 		MessageBox(NULL, TEXT(
 					"JACK is not available.\n"
@@ -1155,18 +1531,22 @@ int main (int argc, char **argv) {
 	rb_atom_to_ui = jack_ringbuffer_create((UPDATE_FREQ_RATIO) * inst->min_atom_bufsiz);
 	rb_atom_from_ui = jack_ringbuffer_create((UPDATE_FREQ_RATIO) * inst->min_atom_bufsiz);
 
-	/* reolve descriptors */
+#ifdef HAVE_LIBLO
+	rb_osc_to_ui = jack_ringbuffer_create((UPDATE_FREQ_RATIO) * inst->nports_ctrl * 2 * sizeof(float));
+#endif
+
+	/* resolve descriptors */
 	plugin_dsp = inst->lv2_descriptor(inst->dsp_descriptor_id);
 	plugin_gui = inst->lv2ui_descriptor(inst->gui_descriptor_id);
 
 	if (!plugin_dsp) {
-		fprintf(stderr, "cannot resolve LV2 descriptor\n");
+		fprintf (stderr, "cannot resolve LV2 descriptor\n");
 		rv |= 2;
 		goto out;
 	}
 	/* jack-open -> samlerate */
 	if (init_jack(extui_host.plugin_human_id)) {
-		fprintf(stderr, "cannot connect to JACK.\n");
+		fprintf (stderr, "cannot connect to JACK.\n");
 #ifdef _WIN32
 		MessageBox (NULL, TEXT(
 					"Cannot connect to JACK.\n"
@@ -1177,6 +1557,8 @@ int main (int argc, char **argv) {
 					"Cannot connect to JACK.\n"
 					"Please start the JACK Server first."
 				);
+#else
+		(void) system ("xmessage -button ok -center \"Cannot connect to JACK.\nPlease start the JACK Server first.\" &");
 #endif
 		rv |= 4;
 		goto out;
@@ -1185,7 +1567,7 @@ int main (int argc, char **argv) {
 	/* init plugin */
 	plugin_instance = plugin_dsp->instantiate(plugin_dsp, j_samplerate, NULL, features);
 	if (!plugin_instance) {
-		fprintf(stderr, "instantiation failed\n");
+		fprintf (stderr, "instantiation failed\n");
 		rv |= 2;
 		goto out;
 	}
@@ -1218,7 +1600,7 @@ int main (int argc, char **argv) {
 				plugin_dsp->connect_port(plugin_instance, p , atom_out);
 				break;
 			default:
-				fprintf(stderr, "yet unsupported port..\n");
+				fprintf (stderr, "yet unsupported port..\n");
 				break;
 		}
 	}
@@ -1242,6 +1624,37 @@ int main (int argc, char **argv) {
 		lv2_atom_forge_init(&lv2_forge, &uri_map);
 	}
 
+	if (dump_ports) {
+		dump_control_ports();
+	}
+
+	// apply user settings
+	for (uint32_t i = 0; i < n_pval; ++i) {
+		if (pval[i].port_idx >= inst->nports_ctrl) {
+			fprintf (stderr, "Invalid Parameter 0 <= %d < %d\n", pval[i].port_idx, inst->nports_ctrl);
+			continue;
+		}
+
+		const uint32_t port_index = portmap_rctl[pval[i].port_idx];
+
+		if (inst->ports[port_index].porttype != CONTROL_IN) {
+			fprintf (stderr, "mapped port (%d) for port %d is not a control input.\n", port_index, pval[i].port_idx);
+			continue;
+		}
+
+		if (inst->ports[port_index].val_min < inst->ports[port_index].val_max) {
+			if (pval[i].value < inst->ports[port_index].val_min || pval[i].value > inst->ports[port_index].val_max) {
+				fprintf (stderr, "Value for port %d out of bounds %f <= %f <= %f\n",
+						pval[i].port_idx,
+						inst->ports[port_index].val_min, pval[i].value, inst->ports[port_index].val_max);
+				continue;
+			}
+		}
+
+		//fprintf (stdout, "Port: %d,  %d -> %f\n", pval[i].port_idx, port_index, pval[i].value);
+		plugin_ports_pre[pval[i].port_idx] = pval[i].value;
+	}
+
 	if (jack_portsetup()) {
 		rv |= 12;
 		goto out;
@@ -1302,6 +1715,12 @@ int main (int argc, char **argv) {
 
 	jack_portconnect(JACK_AUTOCONNECT);
 
+#ifdef HAVE_LIBLO
+	if (osc_port > 0) {
+		start_osc_server (osc_port);
+	}
+#endif
+
 #ifndef _WIN32
 	signal (SIGHUP, catchsig);
 	signal (SIGINT, catchsig);
@@ -1333,6 +1752,7 @@ int main (int argc, char **argv) {
 	}
 
 out:
+	free (pval);
 	cleanup(0);
 #ifdef _WIN32
 	pthread_win32_process_detach_np();
