@@ -1,6 +1,6 @@
 /* setBfree - DSP tonewheel organ
  *
- * Copyright (C) 2013 Robin Gareus <robin@gareus.org>
+ * Copyright (C) 2013-2015 Robin Gareus <robin@gareus.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,6 +32,8 @@
  * We hook into midi-CC function calls and remember the latest value
  */
 
+#define _XOPEN_SOURCE 700
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -46,7 +48,7 @@ struct b_kv {
 };
 
 static void *kvstore_alloc() {
-  struct b_kv *kv = calloc(1, sizeof(struct b_kv));
+  struct b_kv *kv = (struct b_kv *) calloc(1, sizeof(struct b_kv));
   return kv;
 }
 
@@ -77,7 +79,7 @@ static void kvstore_store(void *kvs, const char *key, const char *value) {
   }
   if (!it) {
     /* allocate new terminal node */
-    it = calloc(1, sizeof(struct b_kv));
+    it = (struct b_kv *) calloc(1, sizeof(struct b_kv));
     kv->next = it;
     it = kv;
     it->key = strdup(key);
@@ -114,13 +116,13 @@ void *allocRunningConfig(void) {
   if (!rc) return NULL;
 
   mccc = rc->mrc.mccc = getCCFunctionCount();
-  rc->mrc.mcc = malloc(mccc * sizeof(int));
+  rc->mrc.mcc = (int*) malloc(mccc * sizeof(int));
   if (!rc->mrc.mcc) {
     free(rc);
     return NULL;
   }
 
-  rc->rrc = kvstore_alloc();
+  rc->rrc = (struct b_kv*) kvstore_alloc();
 
   if (!rc->rrc) {
     free(rc->mrc.mcc);
@@ -239,6 +241,8 @@ void initRunningConfig(void *t, void *mcfg) {
   notifyControlChangeByName(mcfg, "pedal.drawbar8", 32);
 
   notifyControlChangeByName(mcfg, "vibrato.routing", 0);
+  notifyControlChangeByName(mcfg, "vibrato.upper", 0);
+  notifyControlChangeByName(mcfg, "vibrato.lower", 0);
   notifyControlChangeByName(mcfg, "vibrato.knob", 0);
 
   notifyControlChangeByName(mcfg, "percussion.enable", 0);
@@ -249,7 +253,7 @@ void initRunningConfig(void *t, void *mcfg) {
   notifyControlChangeByName(mcfg, "overdrive.enable", 0);
   notifyControlChangeByName(mcfg, "overdrive.character", 0);
 
-  notifyControlChangeByName(mcfg, "reverb.mix", 38);
+  notifyControlChangeByName(mcfg, "reverb.mix", 127 * .1);
   notifyControlChangeByName(mcfg, "swellpedal1", 127);
   notifyControlChangeByName(mcfg, "rotary.speed-select", 4*15);
 }
